@@ -5,18 +5,14 @@ from typing import Union, Iterable
 import pandas as pd
 import numpy as np
 import plotly.graph_objects as go
+import os
+from PIL import Image
 
-#from plotly.offline import init_notebook_mode
-
-# to make notebook's plots still active after rebooting the Kernel:
-#import plotly.io as pio
-
-
-#def notebook_offline_settings():
-#    pio.renderers.default = 'notebook'
-
-#    init_notebook_mode(connected=True)
-#    cf.go_offline()
+from jupyter_dash import JupyterDash
+from dash import dcc
+from dash import html
+from dash.dependencies import Input, Output, State
+from dash import callback_context
 
 
 class Heatmap:
@@ -35,7 +31,10 @@ class Heatmap:
                     x_axis_title: str = 'Currents, A',
                     y_axis_title: str = 'Frequencies, Hz',
                     colorbar_text: str = None,
-                    logo: bool = True
+                    logo: bool = False,
+                    logo_local = False,
+                    save_png: str = None,
+                    save_svg: str = None
                     ):
         """
         Built a complete plot of the Two Tone Spectroscopy
@@ -77,8 +76,6 @@ class Heatmap:
         :return:
         """
 
-        #notebook_offline_settings()
-
         if data is not None:
             fig = go.Figure(data=go.Heatmap(
                 z=data.iloc[:, 2],
@@ -109,6 +106,12 @@ class Heatmap:
             bg_clr = '#1c1c1c'
             paper_clr = '#1c1c1c'
             grid_clr = '#757575'
+        elif theme.lower() == 'pure_white':
+            fig.update_layout(template="plotly_white")
+            title_clr = '#000000'
+            bg_clr = '#ffffff'
+            paper_clr = '#ffffff'
+            grid_clr = '#757575'
         else:
             fig.update_layout(template="plotly_dark")
             raise NameError
@@ -155,13 +158,35 @@ class Heatmap:
         else:
             pass
 
+        if logo_local:
+            img = Image.open('logo_sign.png')
+            fig.add_layout_image(
+                dict(
+                    source=img,
+                    xref="paper", yref="paper",
+                    x=1, y=1.09,
+                    sizex=0.09,
+                    sizey=0.09,
+                    xanchor="right", yanchor="bottom",
+                    opacity=1,
+                    layer="above"))
+        else:
+            pass
+
+
         """grid color. See more https://plotly.com/python/axes/"""
         fig.update_layout(xaxis=dict(showgrid=True, gridcolor=grid_clr, gridwidth=0.1,
                                      zeroline=True, zerolinewidth=0.1, zerolinecolor=grid_clr),
                           yaxis=dict(showgrid=True, gridcolor=grid_clr, gridwidth=0.1,
                                      zeroline=True, zerolinewidth=0.1, zerolinecolor=grid_clr))
 
-        fig.show('notebook')
+        if save_png is not None:
+            fig.write_image(save_png)
+
+        if save_svg is not None:
+            fig.write_image(save_svg)
+
+        return fig
 
     @staticmethod
     def decorate_figure(fig, *,
@@ -172,7 +197,8 @@ class Heatmap:
                         x_axis_title: str = 'Currents, A',
                         y_axis_title: str = 'Frequencies, Hz',
                         colorbar_text: str = None,
-                        logo: bool = True,
+                        logo: bool = False,
+                        logo_local=False,
                         y_logo_pos: float = 1.09
                         ):
         """
@@ -212,8 +238,6 @@ class Heatmap:
         :param y_logo_pos: y position of the logo. Default = 1.09. Usually belongs to [0.9, 1.1] interval
         """
 
-        #notebook_offline_settings()
-
         """More figure|plot settings here: https://plotly.com/python/reference/layout/"""
 
         """Adding subtitle"""
@@ -233,6 +257,12 @@ class Heatmap:
             bg_clr = '#1c1c1c'
             paper_clr = '#1c1c1c'
             grid_clr = '#757575'
+        elif theme.lower() == 'pure_white':
+            fig.update_layout(template="plotly_white")
+            title_clr = '#000000'
+            bg_clr = '#ffffff'
+            paper_clr = '#ffffff'
+            grid_clr = '#757575'
         else:
             fig.update_layout(template="plotly_dark")
             raise NameError
@@ -282,8 +312,135 @@ class Heatmap:
         else:
             pass
 
+        if logo_local:
+            img = Image.open('logo_sign.png')
+            fig.add_layout_image(
+                dict(
+                    source=img,
+                    xref="paper", yref="paper",
+                    x=1, y=1.09,
+                    sizex=0.09,
+                    sizey=0.09,
+                    xanchor="right", yanchor="bottom",
+                    opacity=1,
+                    layer="above"))
+        else:
+            pass
+
         """grid color. See more https://plotly.com/python/axes/"""
         fig.update_layout(xaxis=dict(showgrid=True, gridcolor=grid_clr, gridwidth=0.1,
                                      zeroline=True, zerolinewidth=0.1, zerolinecolor=grid_clr),
                           yaxis=dict(showgrid=True, gridcolor=grid_clr, gridwidth=0.1,
                                      zeroline=True, zerolinewidth=0.1, zerolinecolor=grid_clr))
+
+        pass
+
+
+    """
+    info https://plotly.com/python/static-image-export/
+    requirements:
+    conda install -c conda-forge python-kaleido
+    
+    """
+    @staticmethod
+    def save_fig_as(fig, name: str = 'figure.svg', method: str = 'image', create_folder: bool = False, folder_name: str = 'images'):
+        """
+
+        :param fig:
+        :param name: export format options for images: [.svg (default), .pdf, .png, .jpeg, .webp]
+                     HTML export
+        :param method: exporting method [image (default), html]
+        :param create_folder:
+        :param folder_name:
+        :return:
+        """
+        if create_folder == True:
+            if not os.path.exists(folder_name):
+                os.mkdir(folder_name)
+
+            directory = folder_name+'/'+name
+
+        else:
+            directory = name
+
+        if method == 'image':
+            fig.write_image(directory)
+
+        elif method == 'html':
+            fig.write_html(directory)
+
+
+class Dash_app:
+
+    @classmethod
+    def configure_app(cls, data, fig, mode='inline', port=8051, interval=5e3):
+        cls.fig = fig
+        cls.mode = mode
+        cls.interval = interval
+        cls.port = port
+
+        disabled_btn = False
+        #interval_interval = 5e3
+        maxx_interval = -1
+
+        # Build App
+        app = JupyterDash(__name__)
+        app.layout = html.Div(
+            [
+                html.Button('Save CSV', id='btn-nclicks-1', n_clicks=0),
+                html.Button('Save PDF', id='btn-nclicks-2', n_clicks=0),
+                html.Button('Save HTML', id='btn-nclicks-3', n_clicks=0),
+                html.Button('Save SVG', id='btn-nclicks-5', n_clicks=0),
+                dcc.Checklist(id='checkbox',
+                              options=[{'label': 'stop live upd', 'value': 'NO'}],
+                              value=['YES', 'NO'],
+                              labelStyle={'display': 'inline-block'}
+                              ),
+                html.Button('Refresh', id='btn-nclicks-4', n_clicks=0),
+                dcc.Graph(id="heatmap", figure=cls.fig),
+                dcc.Interval(id="animateInterval", interval=cls.interval, n_intervals=0,
+                             max_intervals=maxx_interval),
+            ],
+        )
+
+        @app.callback(
+            Output("heatmap", "figure"),
+            Output("animateInterval", "max_intervals"),
+            Output('btn-nclicks-4', 'disabled'),
+            Input('btn-nclicks-1', 'n_clicks'),
+            Input('btn-nclicks-2', 'n_clicks'),
+            Input('btn-nclicks-3', 'n_clicks'),
+            Input('btn-nclicks-5', 'n_clicks'),
+            Input('checkbox', 'value'),
+            Input('btn-nclicks-4', 'n_clicks'),
+            Input("animateInterval", "n_intervals"),
+        )
+        def doUpdate(btn1, btn2, btn3, btn5, chkbx, btn4, i):
+
+            if chkbx[-1] == 'NO':
+                maxx_interval = 0
+                disabled_btn = False
+            else:
+                maxx_interval = -1
+                disabled_btn = True
+
+            changed_id = [p['prop_id'] for p in callback_context.triggered][0]
+            if 'btn-nclicks-1' in changed_id:
+                data.get_result().to_csv('present_data.csv', index=False)
+            elif 'btn-nclicks-2' in changed_id:
+                fig.write_image('fig1.pdf')
+            elif 'btn-nclicks-3' in changed_id:
+                fig.write_html('fig1.html')
+            elif 'btn-nclicks-5' in changed_id:
+                fig.write_image('fig1.svg')
+
+            elif 'btn-nclicks-4' in changed_id:
+                z = data.njit_result
+                fig.update_traces(z=z)
+            else:
+                pass
+            z = data.njit_result
+
+            return cls.fig.update_traces(z=z), maxx_interval, disabled_btn
+
+        return app.run_server(mode=cls.mode, port=cls.port)
